class User < ActiveRecord::Base
  attr_accessor :password
  before_save :encrypt_password
  has_many :tasks
  has_many :comments, :through => :tasks
  has_many :comments_made, :class_name => "Comment"
  
  validates :username, 
     :presence => true, 
     :uniqueness => true, 
     :length => { :within => 4..18, :too_long => "Username is too long", :too_short => "Username is too short"}
  validates :password, :confirmation => true, :if => :password_required?
  
  def self.encrypt(pass,salt)
    Digest::SHA2.hexdigest(salt+pass)
  end
  
  def encrypt_password
    return if password.blank?
    if new_record?
      self.salt = SecureRandom.hex(16)
    end
    self.encrypted_password = User.encrypt(password, salt)
  end
  
  def self.authenticate(username, pass)
    user = User.find_by(username: username)
    user && user.authenticated?(pass) ? user : nil
  end
  
  def authenticated?(pass)
    encrypted_password == User.encrypt(pass, self.salt)
  end
  
  def password_required?
    encrypted_password.blank? || !self.password.blank?
  end
end
